from assemblyline import odm


class Antivirus(odm.Model):
    class Detection(odm.Model):
        class Engine(odm.Model):
            definition_version = odm.Optional(odm.Keyword())                    # Version of definition set
            name = odm.Keyword()                                                # Name of AV engine
            version = odm.Optional(odm.Keyword())                               # Version of AV engine

        # What category does the verdict fall under?
        category = odm.Optional(odm.Enum(['type-unsupported',                   # File sent to AV is unsupported
                                          'undetected',                         # File not detected by AV
                                          'failure',                            # AV failed during detection
                                          'harmless',                           # AV deems harmless
                                          'suspicious',                         # AV deems suspicious
                                          'malicious']))                        # AV deems malicious
        engine = odm.Compound(Engine)
        virus_name = odm.Optional(odm.Keyword())                                # The name of the virus

    odm_version = odm.Text(default="1.0")                                       # Version of AV ontological result
    detections = odm.List(odm.Compound(Detection))                              # List of AV detections



from assemblyline import odm
from assemblyline.odm.models.alert import Attack
from assemblyline.odm.models.ontology.types.process import Process
from assemblyline.odm.models.ontology.types.network import NetworkConnection, NetworkDNS, NetworkHTTP


# The result ontology for sandbox output
class Sandbox(odm.Model):

    # The metadata of the analysis, per analysis
    class AnalysisMetadata(odm.Model):

        # The metadata regarding the machine where the analysis took place
        class MachineMetadata(odm.Model):
            # The IP of the machine used for analysis
            ip = odm.Optional(odm.IP())
            # The hypervisor of the machine used for analysis
            hypervisor = odm.Optional(odm.Keyword())
            # The name of the machine used for analysis
            hostname = odm.Optional(odm.Keyword())
            # The platform of the machine used for analysis
            platform = odm.Optional(odm.Platform())
            # The version of the operating system of the machine used for analysis
            version = odm.Optional(odm.List(odm.Keyword()))
            # The architecture of the machine used for analysis
            architecture = odm.Optional(odm.Processor())

        # The ID used for identifying the analysis task
        task_id = odm.Optional(odm.Keyword())
        # The start time of the analysis
        start_time = odm.Date()
        # The end time of the analysis
        end_time = odm.Date()
        # The routing used in the sandbox setup (Spoofed, Internet, Tor, VPN)
        routing = odm.Optional(odm.Keyword())
        # The metadata of the analysis
        machine_metadata = odm.Optional(odm.Compound(MachineMetadata))

    # A signature that was raised during the analysis of the task
    class Signature(odm.Model):

        # An indicator of compromise, aka something interesting that the signature was raised on that is worth reporting
        class IOC(odm.Model):
            # An IP that is an indicator of compromise
            ip = odm.Optional(odm.IP())
            # A domain that is an indicator of compromise
            domain = odm.Optional(odm.Domain())
            # An URI that is an indicator of compromise
            uri = odm.Optional(odm.URI())
            # The path of an URI that is an indicator of compromise
            uri_path = odm.Optional(odm.URIPath())
            # A process that is an indicator of compromise
            process = odm.Optional(odm.Compound(Process))

            # TODO: Require ODM models for these values
            # file = odm.Optional(odm.Text())
            # registry = odm.Optional(odm.Text())

        # The process associated with the signature
        process = odm.Optional(odm.Compound(Process))
        # The name of the signature
        name = odm.Keyword()
        # The description of the signature
        description = odm.Optional(odm.Keyword())
        # A list of Att&ck patterns and categories of the signature
        attack = odm.Optional(odm.List(odm.Compound(Attack)))
        # A list of indicators of compromise. A signature can have more than one IOC.
        iocs = odm.Optional(odm.List(odm.Compound(IOC)))

    # Metadata for the analysis
    analysis_metadata = odm.Compound(AnalysisMetadata)
    # Signatures that the file may have
    signatures = odm.Optional(odm.List(odm.Compound(Signature)))
    # The IP traffic observed during analysis
    network_connections = odm.Optional(odm.List(odm.Compound(NetworkConnection)))
    # The DNS traffic observed during analysis
    network_dns = odm.Optional(odm.List(odm.Compound(NetworkDNS)))
    # The HTTP traffic observed during analysis
    network_http = odm.Optional(odm.List(odm.Compound(NetworkHTTP)))
    # A list of processes
    processes = odm.Optional(odm.List(odm.Compound(Process)))
    # The name of the sandbox
    sandbox_name = odm.Keyword()
    # The version of the sandbox
    sandbox_version = odm.Optional(odm.Keyword())
    # Version of AV ontological result
    odm_version = odm.Text(default="1.0")


from assemblyline import odm
from assemblyline.odm.models.ontology.types.process import Process


# Details for a low-level network connection by IP
class NetworkConnection(odm.Model):

    # The GUID associated with the connection
    guid = odm.Text()
    # The process that spawned the network connection
    process = odm.Optional(odm.Compound(Process))
    # The source IP of the connection
    source_ip = odm.IP()
    # The source port of the connection
    source_port = odm.Integer()
    # The destination IP of the connection
    destination_ip = odm.IP()
    # The destination IP of the connection
    destination_port = odm.Integer()
    # The transport layer protocol of the connection
    transport_layer_protocol = odm.Enum(["tcp", "udp"])
    # The direction of the network connection
    direction = odm.Enum(["outbound", "inbound", "unknown"])


# Details for a DNS request
class NetworkDNS(odm.Model):

    # The GUID associated with the connection
    guid = odm.Text()
    # The process that spawned the network connection
    process = odm.Optional(odm.Compound(Process))
    # The low-level details of the DNS request
    connection_details = odm.Compound(NetworkConnection)
    # The domain requested
    domain = odm.IP()
    # A list of IPs that were resolved
    resolved_ips = odm.List(odm.IP())


# Details for an HTTP request
class NetworkHTTP(odm.Model):

    # The GUID associated with the connection
    guid = odm.Text()
    # The process that spawned the network connection
    process = odm.Optional(odm.Compound(Process))
    # The low-level details of the DNS request
    connection_details = odm.Compound(NetworkConnection)
    # The URI requested
    uri = odm.URI()
    # Headers included in the request
    request_headers = odm.Mapping(str)
    # The method of the request
    request_method = odm.Enum(["GET", "POST", "PUT", "DELETE", "HEAD", "CONNECT", "OPTIONS", "TRACE", "PATCH"])
    # The status code of the response
    response_status_code = odm.Optional(odm.Integer())



from assemblyline import odm


# Details about a process
class Process(odm.Model):

    # The GUID associated with the process
    guid = odm.Text()

    # Parent process details
    # The GUID associated with the parent process
    pguid = odm.Text()
    # The image of the parent process that spawned this process
    pimage = odm.Text()
    # The command line that the parent process ran
    pcommand_line = odm.Text()
    # The process ID of the parent process
    ppid = odm.Integer()

    # The process ID
    pid = odm.Integer()
    # The image of the process
    image = odm.Text()
    # The command line that the process ran
    command_line = odm.Text()
    # The time of creation for the process
    start_time = odm.Date()
    # The time of termination for the process
    end_time = odm.Date()
